/*
 *  PostgresDataStore.java
 *
 *  Copyright (c) 1998-2001, The University of Sheffield.
 *
 *  This file is part of GATE (see http://gate.ac.uk/), and is free
 *  software, licenced under the GNU Library General Public License,
 *  Version 2, June 1991 (in the distribution as file licence.html,
 *  and also available at http://gate.ac.uk/gate/licence.html).
 * 
 *  Valentin Tablan 21 Feb 2000
 *
 *  $Id: PostgresDataStore.java,v 1.5 2000/11/08 16:34:56 hamish Exp $
 */
 
package gate.db;

import java.sql.*;
import gate.*;
import gate.util.*;
import java.util.*;
import java.io.*;
import postgresql.largeobject.*;
import postgresql.fastpath.*;

/** Provides connection to an Oracle Database.
  */
public class PostgresDataStore extends Database{

  /** Debug flag */
  private static final boolean DEBUG = false;

  /**
   * Constructs a OracleDataStore
   *
   * @param driver the driver used to connect to the database.
   * We didn't choose to default this value to any specific Oracle Jdbc
   * drivers because this option should belong to the user.
   * @param url The URL of the database.
   * @param user the username
   * @param password the password
   * @param catalog the catalog in which the Gate2 table reside.
   * @param schema The schema in which the Gate2 table reside.
   */
  public PostgresDataStore(Driver driver, String url, String user,
                         String password, String catalog, String schema)
         throws SQLException
  {
    super(driver, url, user, password, catalog, schema);
    dictionary.put("BLOB", "OID");
    dictionary.put("FLOAT", "FLOAT4");
    dictionary.put("DOUBLE", "FLOAT8");
    dictionary.put("LONG", "INT8");
  } // PostgresDataStore

  /** Reads a Object from a BLOB on the current row in a result set.
    * Note that the Blob must contain a java Object saved using serialisation.
    * @param rs a result set positioned on the row from which the BLOB is to
    * be read
    * @param columnName the name of the column containing the BLOB on the
    * current row.
    * @return an object containing the blob.
    */
  public Object readObjectBlob(ResultSet rs, Connection con, String columnName)
                throws SQLException, IOException{

    Object ret = null;
    int oid =  rs.getInt(columnName);
    lobMan = ((postgresql.Connection)con).getLargeObjectAPI();
    LargeObject lob = lobMan.open(oid,LargeObjectManager.READ);
    int size = lob.size();
    byte buff[] = new byte[size];

    lob.read(buff,0,size);
    ObjectInputStream ois = new ObjectInputStream(
                                new ByteArrayInputStream(buff));
    try {
      ret = ois.readObject();
    } catch (ClassNotFoundException cnfe) {
      cnfe.printStackTrace(Err.getPrintWriter());
    }
    ois.close();
    lob.close();

    /*
    ObjectWriter ow = new ObjectWriter(value);
    int size = ow.getSize();
    buff = new byte[size];
    ow.start();
    InputStream is = ow.getInputStream();
    is.read(buff,0,size);
    lob.write(buff, 0, size);
    is.close();
    lob.close();



    Blob objBlob = rs.getBlob(columnName);
    ObjectInputStream ois=null;
    InputStream is=null;
    if(objBlob !=null){
      try{
        is = objBlob.getBinaryStream();
        if(is != null) ois = new ObjectInputStream(is);
      }catch(StreamCorruptedException sce){
        sce.printStackTrace(Err.getPrintWriter());
      }catch(IOException ioe){
        ioe.printStackTrace(Err.getPrintWriter());
      };
      if(ois != null){
        try{
          ret = ois.readObject();
          is.close();
          ois.close();
        }catch(ClassNotFoundException cnfe){
          cnfe.printStackTrace(Err.getPrintWriter());
        }catch(IOException ioe){
          ioe.printStackTrace(Err.getPrintWriter());
        };
      };//if (ois != null)
    }else{
      ret = null;
    }//if(objBlob !=null)
    */
    return ret;

    ////////////////////// OLD CODE ////////////////////////
    //don't feel like throwing it away yet
    /*
        //the generic JDBC way ... doesn't seem to work with ORACLE
        ObjectInputStream ois=null;
        try{
          InputStream is = rsCorpora.getBinaryStream("FEATURES");
          if(is != null){
            ois = new ObjectInputStream(is);
          }else{
            ois = null;
          }
        }catch(StreamCorruptedException sce){
          sce.printStackTrace(Err.getPrintWriter());
        }catch(IOException ioe){
          ioe.printStackTrace(Err.getPrintWriter());
        };
        if(ois != null){
          try{
            features = (FeatureMap)ois.readObject();
          }catch(ClassNotFoundException cnfe){
            cnfe.printStackTrace(Err.getPrintWriter());
          }catch(IOException ioe){
            ioe.printStackTrace(Err.getPrintWriter());
          };
        is.close();
        ois.close();
        };//if (ois != null)
    */
    /*
        //the ORACLE way
        oracle.sql.BLOB featuresBlob =
            ((oracle.jdbc.driver.OracleResultSet)rsCorpora).getBLOB("FEATURES");
        ObjectInputStream ois=null;
        InputStream is=null;
        if(featuresBlob !=null){
          try{
            is = featuresBlob.getBinaryStream();
            if(is != null) ois = new ObjectInputStream(is);
          }catch(StreamCorruptedException sce){
            sce.printStackTrace(Err.getPrintWriter());
          }catch(IOException ioe){
            ioe.printStackTrace(Err.getPrintWriter());
          };
          if(ois != null){
            try{
              features = (FeatureMap)ois.readObject();
              is.close();
              ois.close();
            }catch(ClassNotFoundException cnfe){
              cnfe.printStackTrace(Err.getPrintWriter());
            }catch(IOException ioe){
              ioe.printStackTrace(Err.getPrintWriter());
            };
          };//if (ois != null)
        }else{
          features = null;
        }//if(featuresBlob !=null)
    */
  }

  /**
  * Acts as a setBlob for the provided prepared statement on the position
  * indicated by place. Uses Postgresql extensions.
  * @param ps A prepared statement where the blob will be set.
  * @param place The position on the prepared statement where the blob
  * will be set.
  * @param value The object that will be serialised and converted into a blob.
  */
  public void putObject(PreparedStatement ps, int place, Object value)
    throws SQLException, IOException
  {
    /*
    postgresql.jdbc2.ResultSet rs;
    postgresql.util.Serialize.create((postgresql.Connection)ps.getConnection(),
                                      value);
    postgresql.util.Serialize ser = new postgresql.util.Serialize(
                                    (postgresql.Connection)ps.getConnection(),
                                    value.getClass().toString());
    ser.create((postgresql.Connection)ps.getConnection(),
                                      value.getClass());
    ps.setObject(place, value);

    Fastpath fp = ((postgresql.Connection)ps.getConnection()).getFastpathAPI();
    */
    lobMan = ((postgresql.Connection)ps.getConnection()).getLargeObjectAPI();
    int oid = lobMan.create(LargeObjectManager.WRITE);
    LargeObject lob = lobMan.open(oid,LargeObjectManager.WRITE);

    byte buff[];
    ObjectWriter ow = new ObjectWriter(value);
    int size = ow.getSize();
    buff = new byte[size];
    ow.start();
    InputStream is = ow.getInputStream();
    is.read(buff,0,size);
    lob.write(buff, 0, size);
    is.close();
    lob.close();

    /*
    ObjectOutputStream oos = new ObjectOutputStream(lob.getOutputStream());
    oos.writeObject(value);
    oos.close();
    */
    ps.setInt(place, oid);

  } // putObject


  public Map getDictionary(){
    return dictionary;
  } // getDictionary

  protected Map dictionary = new HashMap();

  LargeObjectManager lobMan;

} // PostgresDataStore


